from Products.Archetypes.public import BaseFolderSchema, Schema
from Products.Archetypes.public import BaseFolder, registerType

from Products.CMFFolder.config import *

from Products.Archetypes.public import BooleanField, FixedPointField, IntegerField, LinesField, StringField, TextField
from Products.Archetypes.public import BooleanWidget, DecimalWidget, IntegerWidget, LinesWidget, SelectionWidget, StringWidget, TextAreaWidget


from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo

from Products.CMFCore.utils import getToolByName

from AccessControl import getSecurityManager, SpecialUsers
from Acquisition import aq_base, aq_parent

from Permissions import *
import string


#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseFolderSchema +  Schema((
    LinesField('left_slots',
              mode='r',
              accessor='left_slots'
              ),
    LinesField('custom_left_slots',
              widget=LinesWidget(
                  label="left slots",
                  description="leave blank if you want to acquire setting; type 'empty' if you do not use any slot.",
                  ),
              ),
    LinesField('right_slots',
              mode='r',
              accessor='right_slots'
              ),
    LinesField('custom_right_slots',
              widget=LinesWidget(
                  label="right slots",
                  description="leave blank if you want to acquire setting; type 'empty' if you do not use any slot.",
                  ),
              ),
    BooleanField ('is_allow_nested',
              write_permission=SetQuota,
              default='1',
              mutator='setIs_allow_nested',
              widget=BooleanWidget(label="Allow Nested",
                  description="Allow creating CMFFolder insider another CMFFolder"
                 ),
        ),
    ))


class CMFFolder(BaseFolder):
    """
        CMFFolder has quota count and assistant tools
    """
    security = ClassSecurityInfo()
    
    schema = schema
    archetype_name = "CMF Folder"
    
    #allow_discussion = 1
    
    global_allow = 1
    
    
    
    filter_content_types = 0
    security = ClassSecurityInfo()

    
    actions = (
          { 'id': 'folderContents'
               ,'name': 'Contents'
               ,'action': 'string: ${folder_url}/folder_contents'
               ,'condition': 'python:member and folder is not object and object.displayContentsTab()'
               ,'permissions': (CMFCorePermissions.ListFolderContents,)
               , 'category'      : 'object'
               , 'visible'       : 1
          },
          {'id': 'view',
                'name': 'View',
                'action': 'string:folder_listing',
                'permissions': (CMFCorePermissions.View,),
                'category'      : 'object',
                'visible'       : 1 },
          { 'id'            : 'local_roles'
              , 'name'          : 'Sharing'
              , 'action'        : 'string:${folder_url}/folder_localrole_form'
              , 'permissions'   :
                 (CMFCorePermissions.ManageProperties,)
              , 'category'      : 'folder'
              , 'visible'       : 1
          },
          { 'id': 'folderlisting'
               ,'name': 'Folder Listing'
               ,'action': 'string:${folder_url}/folder_listing'
               ,'permissions': (CMFCorePermissions.View,)
               , 'category'      : 'folder'
               , 'visible'       : 0
          },
          { 'id': 'content_status_history'
               ,'name': 'State'
               ,'action': 'string:$object_url/content_status_history'
               ,'condition': 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())'
               ,'permissions': (CMFCorePermissions.View,)
               , 'category'      : 'object_tabs'
               , 'visible'       : 0
          })


    security.declareProtected(CMFCorePermissions.View, 'getIcon')
    def getIcon(self, relative_to_portal=0):
        return 'cmffolder_icon.gif'
        

    security.declareProtected( CMFCorePermissions.View, 'left_slots' )        
    def left_slots(self):
        val = self.getCustom_left_slots()
        if not val:
            parent = self.aq_parent
            if hasattr(parent, 'left_slots'):
                ls = parent.left_slots
                if callable(ls):
                    return ls()
                else:
                    return ls
            else:
                return []
        elif 'empty' in val:
            return []
        else:
            return val
    
    security.declareProtected( CMFCorePermissions.View, 'right_slots' )        
    def right_slots(self):
        val = self.getCustom_right_slots()
        if not val:
            parent = self.aq_parent
            if hasattr(parent, 'right_slots'):
                rt = parent.right_slots
                if callable(rt):
                    return rt()
                else:
                    return rt
            else:
                return []
        elif 'empty' in val:
            return []
        else:
            return val
 
 
    ##### Mutators #####
    
    ### Checking functions ####
    security.declarePublic('check_nested')
    def check_nested(self):
        path=self.absolute_url('relative')
        path=path.replace('%20', ' ')
        items= self.portal_catalog(meta_type='CMFFolder', path=path)
        if len(items)>1:   #self is included
            return 1
        else:
            return 0
            
            
    def manage_afterAdd(self, item, container):
        # print "afterAdd %s %s"%(str(item), str(container))
        parent = self.aq_parent
        while hasattr(parent, 'aq_parent'):
            if hasattr(parent.aq_explicit, 'getIs_allow_nested'):
                allow_nested= parent.aq_explicit.getIs_allow_nested()
                if not allow_nested:
                    raise CMFFolderNestedException, \
                      "Cannot add a CMFFolder inside this non-nestable CMFFolder"
            parent = parent.aq_parent
            
        # maybe we can check_over_size and check_over_count here
        
        BaseFolder.manage_afterAdd(self, item, container)
          


registerType(CMFFolder)
