#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, package_home
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.CMFCorePermissions  import ModifyPortalContent, ManagePortal
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFFolder.config import *

from Acquisition import aq_parent, aq_base, aq_chain

class CMFFolderTool(UniqueObject, SimpleItem):
    """ CMFFolder Tool  """ 

    id        = 'cmffolder_tool'
    meta_type = 'CMFFolder Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    manage_options=(
        ({ 'label'   : 'Config',
           'action'  : 'manage_configForm',
           },
         ) + SimpleItem.manage_options
        )

    security.declareProtected( ManagePortal
                             , 'manage_configForm' )    
    
    manage_configForm = PageTemplateFile('www/config', globals(), __name__='manage_configForm')

    def __init__(self):
        self._reset()

    def _reset(self):
        
        self.MAX_QUOTA_BYTES	= 0
        self.MAX_QUOTA_OBJECTS	= 0
        self.MAX_OBJECT_SIZE	= 0
        self.IS_ALLOW_NESTED	= 1
        self.QUOTA_CATEGORIES = ['Basic:all']
        self.APPLY_SUB_FOLDERISH	= 0
        
        self.QUOTACHECKING_FOLDER_PATHS = []
        
        self.OBJECT_CSS={}
        
        self.DISABLE_FOLDERISH_SETTING	= 0
        return

    def _set(self, REQUEST=None, **kwargs):
        if REQUEST==None:
            arglist= kwargs
        else:
            arglist = REQUEST
        
        self.MAX_QUOTA_BYTES	= arglist['MAX_QUOTA_BYTES']
        self.MAX_QUOTA_OBJECTS	= arglist['MAX_QUOTA_OBJECTS']
        self.MAX_OBJECT_SIZE	= arglist['MAX_OBJECT_SIZE']
        self.IS_ALLOW_NESTED	= arglist['IS_ALLOW_NESTED']
        self.QUOTA_CATEGORIES	= arglist['QUOTA_CATEGORIES']
        
        self.APPLY_SUB_FOLDERISH	= arglist['APPLY_SUB_FOLDERISH']
        self.QUOTACHECKING_FOLDER_PATHS	= arglist['QUOTACHECKING_FOLDER_PATHS']
        self.DISABLE_FOLDERISH_SETTING	= arglist['DISABLE_FOLDERISH_SETTING']
        
    def _resetStyle(self):
        self.OBJECT_CSS = {}
    
    
    def _updateAllFolders(self):
        
        items= self.portal_catalog(meta_type='QuotaAW')
        for i in items:
            obj = i.getObject()
            obj.setMax_quota_bytes(self.MAX_QUOTA_BYTES, force=1)
            obj.setMax_quota_objects(self.MAX_QUOTA_OBJECTS, force=1)
            obj.setMax_object_size(self.MAX_OBJECT_SIZE, force=1)
            
            obj.setIs_allow_nested(self.IS_ALLOW_NESTED)
            obj.setQuota_categories(self.QUOTA_CATEGORIES)
            obj.setApply_sub_folderish(self.APPLY_SUB_FOLDERISH)
            
    
    def manage_configfolder(self, submit, \
                         REQUEST=None, **kwargs):
    
        """config this puppy"""
        if submit == "Update All QuotaAW":
            self._set(REQUEST=REQUEST, **kwargs)
            self._updateAllFolders()
        elif submit == "Save Quota Config":
            self._set(REQUEST=REQUEST, **kwargs)
        elif submit == "Reset Style":
            self._resetStyle()
        elif submit == "Reset Config":
            self._reset()

        if self.DISABLE_FOLDERISH_SETTING:
                self._updateAllFolders()

        if REQUEST:
            return REQUEST.RESPONSE.redirect(self.absolute_url() + \
                                             "/manage_configForm?portal_status_message=Config+Updated")
    
    ##### Utils #####
    ### return a category list in the following format: ['Member', 'john', 'mary'....]
    
    
    security.declarePublic('getQuota_categories' )
    def getQuota_categories(self, folder):
        quota = self.getQuotaAW(folder)
        return quota.getQuota_categories()
        
    security.declarePublic('getMax_quota_bytes' )
    def getMax_quota_bytes(self, folder):
        quota = self.getQuotaAW(folder)
        return quota.getMax_quota_bytes()
        
        
    security.declarePublic('getMax_quota_objects' )
    def getMax_quota_objects(self, folder):
        quota = self.getQuotaAW(folder)
        return quota.getMax_quota_objects()
        
    security.declarePublic('getMax_object_size' )
    def getMax_object_size(self, folder):
        quota = self.getQuotaAW(folder)
        return quota.getMax_object_size()
        

    security.declarePublic('splitCategory' )
    def splitCategory(self, category):
        l = category.split(':',1)
        r = l[1].split(',')
        if type(r)== type('abc'):
            r= [r]
            
        return [l[0].strip()] + r

    security.declarePublic('filterCategory' )
    def filterCategory(self, folder, items, category='', lists=[]):
        ret = []
        category=category.lower()
        if not category:
            for i in items:
                o = i.getObject()
                if o <> aq_base(folder) and o.meta_type != 'QuotaAW':
                    ret.append(i)
                    
        elif category=='basic':
            for i in items:
                o = i.getObject()
                if o <> aq_base(folder) and o.meta_type != 'QuotaAW':
                    ret.append(i)
        elif category=='creators':
            if 'all' in lists:
                for i in items:
                    o = i.getObject()
                    if o <> aq_base(folder) and o.meta_type != 'QuotaAW':
                        ret.append(i)
            else:
                for i in items:
                    o = i.getObject()
                    if o.Creator() in lists and o <> aq_base(folder) and o.meta_type != 'QuotaAW':
                        ret.append(i)
                        
        #elif category=='owners':
        #    if 'all' in lists:
        #        for i in items:
        #            if i.getObject() <> aq_base(folder):
        #                ret.append(i)
        #    else:
        #        for i in items:
        #            o = i.getObject()
        #            if o.getOwner() in lists and o <> aq_base(folder):
        #                ret.append(i)
        return ret
    
    ##### Accessor #####
    security.declarePublic('getCurrentCount' )
    def getCurrentCount(self, folder, category='', lists=[]):
        path=folder.absolute_url('relative')
        path=path.replace('%20', ' ')
        items= self.portal_catalog(path=path)
        items= self.filterCategory(folder, items,  category, lists)
        #return len(items) -1  #(minus self) 
        # no need to minus self, since it is filtered in the filterCategory function
        return len(items)


    security.declarePublic('getCurrentMaxObjectPath')
    def getCurrentMaxObjectPath(self, folder, category='', lists=[]):
        path=folder.absolute_url('relative')
        path=path.replace('%20', ' ')
        items= self.portal_catalog(path=path )
        items= self.filterCategory(folder, items, category, lists)
        maxsize=0
        maxobjpath =''
        for i in items:
            o = i.getObject()
            if o !=  folder:
                if hasattr(o, 'get_size'):
                    size =o.get_size()
                
                    if size > maxsize:
                        maxsize=size
                        maxobjpath = o.absolute_url('relative')
        
        return maxobjpath.replace('%20', ' ')
    
    security.declarePublic('getCurrentSize' )
    def getCurrentSize(self, folder, category='', lists=[]):
        path = folder.absolute_url('relative')
        path=path.replace('%20', ' ' )
        items= self.portal_catalog(path= path)
        items= self.filterCategory(folder, items, category, lists)
        sum = 0
        for i in items:
            o = i.getObject()
            if o !=  folder:
                if hasattr(o, 'get_size'):
                    sum = sum +  o.get_size()
                
        return sum
        
    ### Checking functions ####
    security.declarePublic('check_over_count' )
    def check_over_count(self, quotaaw, category='', lists=[]):
        quota = quotaaw.getMax_quota_objects()
        if not quota:
            return 0
        quota = int(quota)
        
        return self.getCurrentCount( aq_parent(quotaaw), category=category, lists=lists) > quota

    security.declarePublic('check_over_size' )
    def check_over_size(self, quotaaw, category='', lists=[]):
        
        quota = quotaaw.getMax_quota_bytes()
        if not quota:
            return 0
        quota = int(quota)
        return self.getCurrentSize( aq_parent(quotaaw), category=category, lists=lists) > quota

    security.declarePublic('check_object_over_size' )
    def check_object_over_size(self, quotaaw, category='', lists=[]):
        quota = quotaaw.getMax_object_size()
        if not quota:
            return 0
        quota = int(quota)
        path = self.getCurrentMaxObjectPath(aq_parent(quotaaw), category=category, lists=lists)
        if not path:
            return 0
        obj = self.restrictedTraverse(path)
        
        return obj.get_size() > quota

    security.declarePublic( 'getFileSizeStr' )
    def getFileSizeStr(self, bytes):
    
        try:
            
            """
            Format a number of bytes as bytes, K, or MB.
            """
            megs=bytes / 1048576.0
            if megs > 1:
                return "%.2f MB" % megs
            k=bytes / 1024.0
            if k > 1:
                return "%d K" % k
            return "%d bytes" % bytes
        except:
            return None

    security.declarePublic('getCheckFolder')
    def getCheckFolder(self, item):
        tool = self.cmffolder_tool
        url = self.portal_url
        portalpath = url.getPortalPath()
        shadowparent = None
        
        for parent in item.aq_chain:
            
            if not parent:
                return None
            if not hasattr(parent, 'meta_type'):
                return None
            if parent.meta_type =='QuotaAW' or not parent.isPrincipiaFolderish:
                continue
                
            if not shadowparent and parent.isPrincipiaFolderish:
                shadowparent = parent
            
            if not hasattr(parent, 'contentValues'):
                return None
            quotas = parent.contentValues(filter={'portal_type':'QuotaAW',})
            if len(quotas)>0:
                if quotas[0].getApply_sub_folderish():    # grab from CMFFolder return first parent if apply sub folderish
                    return shadowparent
                else: 
                    return parent
            
            #if hasattr(parent, 'getPhysicalPath') and '/'.join(parent.getPhysicalPath()) in checkpaths:
            #    if tool.APPLY_SUB_FOLDERISH:    # grab from tool return first parent if apply sub folderish
            #        return shadowparent
            #    else: 
            #        return parent
            #if hasattr(parent, 'getPhysicalPath') and '/'.join(parent.getPhysicalPath())==portalpath:
            #    return None
            
        return None


    security.declarePublic('getCheckFolder')
    def getQuotaAW(self, item):
        parent = item
        for parent in item.aq_chain:
            
            quotas = parent.contentValues(filter={'portal_type':'QuotaAW',})
            if len(quotas)>0:
                return quotas[0]
        return None

    ##### set style #####
    security.declareProtected( ModifyPortalContent, 'setObjectStyle' )    
    def setObjectStyle(self, contentpath, csspath):
        #cssstring = contentpath +'::style::' + csspath
        #if cssstring not in self.OBJECT_CSS:
        #    self.OBJECT_CSS.append(cssstring)
        
        contentpath = contentpath.replace('%20', ' ')
        if not csspath:
            del self.OBJECT_CSS[contentpath]
        else:
            self.OBJECT_CSS[contentpath]=csspath.replace('%20', ' ')
    
    ##### set style #####
    security.declarePublic('getObjectStyle' )    
    def getObjectStyle(self, contentpath, relative=1):
        try:
            contentpath=self.OBJECT_CSS[contentpath].replace('%20', ' ')
            css =self.restrictedTraverse(contentpath)
            return css.absolute_url('relative').replace('%20', ' ')
        except:
            return None
            
        
        


InitializeClass(CMFFolderTool)
