from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import BaseContent, registerType

from Products.CMFFolder.config import *

from Products.Archetypes.public import BooleanField, FixedPointField, IntegerField, LinesField, StringField, TextField
from Products.Archetypes.public import BooleanWidget, DecimalWidget, IntegerWidget, LinesWidget, SelectionWidget, StringWidget, TextAreaWidget


from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName

from AccessControl import getSecurityManager, SpecialUsers
from Acquisition import aq_base, aq_parent

from Permissions import *
import string


#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseSchema +  Schema((
    StringField('id',
                mode='r',
                ),
    IntegerField('max_quota_bytes',
              default='0',
              write_permission=SetQuota,
              mutator='setMax_quota_bytes',
              widget=IntegerWidget(
                  label="Quota Size in Bytes",
                  description="(0 for no limit)",
                  ),
              ),
    IntegerField('max_quota_objects',
              default='0',
              write_permission=SetQuota,
              mutator='setMax_quota_objects',
              widget=IntegerWidget(
                  label="Maximum number of Objects",
                  description="(0 for unlimited)",
                  ),
              ),
    IntegerField('max_object_size',
              default='0',
              write_permission=SetQuota,
              mutator='setMax_object_size',
              widget=IntegerWidget(
                  label="Maximum Object Size in Bytes",
                  description="(0 for unlimited)",
                  ),
              ),
    LinesField('quota_categories',
              default=['Basic:all'],
              write_permission=SetQuota,
              mutator='setQuota_categories',
              widget=LinesWidget(label="Quota Categories",
                         description='You can cateogrize the quota counting, quota will be counted for each category' \
                                     'e.g. "Creators:mary,john" will only check Mary\'s and John\'s objects respectively. '\
                                     'Now, only "Creators" category is available. Default is "Basic:all" which did not filter any object.'),
              ),
    BooleanField ('apply_sub_folderish',
              write_permission=SetQuota,
              default='0',
              mutator='setApply_sub_folderish',
              widget=BooleanWidget(label="Apply Quota To All subfolders",
                  description="Apply the quota setting to all folderish contents inside"
                 ),
        ),
    ))


class QuotaAW(BaseContent):
    """
        Quota AnyWhere
    """
    security = ClassSecurityInfo()
    
    schema = schema
    archetype_name = "QuotaAW"
    
    global_allow = 1
    
    security = ClassSecurityInfo()

    
    security.declareProtected(CMFCorePermissions.View, 'getIcon')
    def getIcon(self, relative_to_portal=0):
        return 'quotaaw_icon.gif'
        

    
    ##### Mutators #####
    security.declareProtected( SetQuota, 'setMax_quota_bytes' ) 
    def setMax_quota_bytes(self, value, **kwargs):
        tool = self.cmffolder_tool
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.MAX_QUOTA_BYTES)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
            
        schema = self.Schema()
        field = schema.get('max_quota_bytes', 0)
        field.set(instance=self, value=value)
    
    security.declareProtected( SetQuota, 'setMax_quota_objects')
    def setMax_quota_objects(self, value, **kwargs):
        
        tool = self.cmffolder_tool
        # check if cmffolder_tool disable folderish setting
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.MAX_QUOTA_OBJECTS)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
        schema = self.Schema()
        field = schema.get('max_quota_objects', 0)
        field.set(instance=self, value=value)

    security.declareProtected( SetQuota, 'setMax_object_size' )        
    def setMax_object_size(self, value, **kwargs):
        # check if cmffolder_tool disable folderish setting
        tool = self.cmffolder_tool
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.MAX_OBJECT_SIZE)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
        schema = self.Schema()
        field = schema.get('max_object_size', 0)
        field.set(instance=self, value=value)
    
    
    security.declareProtected( SetQuota, 'setIs_allow_nested' )        
    def setIs_allow_nested(self, value, **kwargs):
        # check if cmffolder_tool disable folderish setting
        tool = self.cmffolder_tool
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.IS_ALLOW_NESTED)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
        
        # check logically possible
        if value not in [1,'1']:
            if self.check_nested():
                 raise CMFFolderNestedException, \
                      "You has nested CMFFolder within this folder, please remove it first before setting the folder to non-nestable."

        schema = self.Schema()
        field = schema.get('is_allow_nested', None)
        field.set(instance=self, value=value)
    
    
    
    security.declareProtected( SetQuota, 'setQuota_categories' )        
    def setQuota_categories(self, value, **kwargs):
        # check if cmffolder_tool disable folderish setting
        tool = self.cmffolder_tool
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.QUOTA_CATEGORIES)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
        
        
        schema = self.Schema()
        field = schema.get('quota_categories', None)
        field.set(instance=self, value=value)
    
    security.declareProtected( SetQuota, 'setApply_sub_folderish' )        
    def setApply_sub_folderish(self, value, **kwargs):
        # check if cmffolder_tool disable folderish setting
        tool = self.cmffolder_tool
        if not kwargs.has_key('force'):
            if value and (str(value) <> str(tool.APPLY_SUB_FOLDERISH)):
                if tool.DISABLE_FOLDERISH_SETTING:
                    raise CMFFolderSetQuotaException, \
                              "Folder's setting violate the cmffolder_tool setting. You are not permitted to change the quota setting."
        
        
        schema = self.Schema()
        field = schema.get('apply_sub_folderish', None)
        field.set(instance=self, value=value)
        
    def manage_afterAdd(self, item, container):
        parent = self.aq_parent
        quotas = parent.contentValues(filter={'portal_type':'QuotaAW',})
        if len(quotas) >1:
            raise QuotaAWObjectException, \
                      "A folder cannot have more than one QuotaAW"
        BaseContent.manage_afterAdd(self, item, container)
        
        
          


registerType(QuotaAW)
